@@ -26,15 +26,16 @@ brk_quantiles <- function (probs, ...) {
     qs <- qs[non_dupes]
     probs <- probs[non_dupes]
 
-    breaks <- create_lr_breaks(qs, left, close_end)
+    breaks <- create_lr_breaks(qs, left)
 
-    needs <- needs_extend(breaks, x, extend)
+    needs <- needs_extend(breaks, x, extend, left, close_end)
     if ((needs & LEFT) > 0)  probs <- c(0, probs)
     if ((needs & RIGHT) > 0) probs <- c(probs, 1)
-    breaks <- maybe_extend(breaks, x, extend)
+    breaks <- extend_and_close(breaks, x, extend, left, close_end)
 
     class(breaks) <- c("quantileBreaks", class(breaks))
     attr(breaks, "scaled_endpoints") <- probs
+    names(breaks) <- names(probs)
 
     breaks
   }
@@ -47,7 +48,7 @@ brk_quantiles <- function (probs, ...) {
 #' @order 2
 brk_equally <- function (groups) {
   assert_that(is.count(groups))
-  brk_quantiles(seq(0, groups)/groups)
+  brk_quantiles(seq(0L, groups)/groups)
 }
 
 
@@ -83,12 +84,12 @@ brk_mean_sd <- function (sds = 1:3, sd = deprecated()) {
     sds <- sort(sds)
     sds <- c(-rev(sds), 0, sds)
     breaks <- sds * x_sd + x_mean
-    breaks <- create_lr_breaks(breaks, left, close_end)
+    breaks <- create_lr_breaks(breaks, left)
 
-    needs <- needs_extend(breaks, x, extend)
+    needs <- needs_extend(breaks, x, extend, left, close_end)
     if ((needs & LEFT) > 0) sds <- c(-Inf, sds)
     if ((needs & RIGHT) > 0) sds <- c(sds, Inf)
-    breaks <- maybe_extend(breaks, x, extend)
+    breaks <- extend_and_close(breaks, x, extend, left, close_end)
 
     class(breaks) <- c("sdBreaks", class(breaks))
     attr(breaks, "scaled_endpoints") <- sds
@@ -112,8 +113,7 @@ brk_pretty <- function (n = 5, ...) {
       return(empty_breaks())
     }
 
-    breaks <- create_lr_breaks(breaks, left, close_end)
-    breaks <- maybe_extend(breaks, x, extend)
+    breaks <- create_extended_breaks(breaks, x, extend, left, close_end)
 
     breaks
   }
@@ -189,76 +189,13 @@ brk_width.default <- function (width, start) {
 
     if (sign(width) <= 0) breaks <- rev(breaks)
 
-    breaks <- create_lr_breaks(breaks, left, close_end)
-    breaks <- maybe_extend(breaks, x, extend)
+    breaks <- create_extended_breaks(breaks, x, extend, left, close_end)
 
     breaks
   }
 }
 
 
-#' Return a sequence of width `width`
-#'
-#' @param width An object representing a width
-#' @param start Element to start from
-#' @param until Result must be just long enough to cover this element
-#'
-#' @return A sequence of breaks
-#' @noRd
-sequence_width <- function(width, start, until) {
-  UseMethod("sequence_width")
-}
-
-
-#' @export
-sequence_width.default <- function (width, start, until) {
-  breaks <- seq(start, until, width)
-
-  too_short <- if (sign(width) > 0) {
-    breaks[length(breaks)] < until
-  } else {
-    breaks[length(breaks)] > until
-  }
-
-  # length(breaks) == 1L captures when start == max_x
-  if (too_short || length(breaks) == 1L) {
-    breaks <- c(breaks, breaks[length(breaks)] + width)
-  }
-
-  breaks
-}
-
-
-#' @export
-sequence_width.Period <- function(width, start, until) {
-  loadNamespace("lubridate")
-
-  if (as.numeric(until - start) %% as.numeric(width) != 0 || until == start) {
-    # extend to cover all data / ensure at least one interval
-    until <- lubridate::add_with_rollback(until, width)
-  }
-  # alternative to seq, using Period arithmetic
-  # We find the number n of widths that gets beyond seq_end
-  # and add (width * 0:n) to start
-  # normally this would be ceiling((seq_end - start)/width)
-  # we calculate it roughly using a Duration
-  n_intervals <- ceiling((until - start)/lubridate::as.duration(width))
-  breaks <- lubridate::add_with_rollback(start, (seq(0, n_intervals) * width))
-
-  last_break <- breaks[length(breaks)]
-  too_short <- if (width > 0) {
-     last_break < until
-  } else {
-    last_break > until
-  }
-  if (too_short) {
-    breaks <- c(breaks, lubridate::add_with_rollback(last_break, width))
-  }
-
-  breaks
-}
-
-
 #' @rdname chop_evenly
 #' @export
 #' @order 2
@@ -271,8 +208,9 @@ brk_evenly <- function(intervals) {
     if (sign(max_x - min_x) <= 0) return(empty_breaks())
 
     breaks <- seq(min_x, max_x, length.out = intervals + 1L)
-    breaks <- create_lr_breaks(breaks, left, close_end)
-    maybe_extend(breaks, x, extend)
+    breaks <- create_extended_breaks(breaks, x, extend, left, close_end)
+
+    breaks
   }
 }
 
@@ -292,14 +230,16 @@ brk_proportions <- function(proportions) {
     if (sign(range_x) <= 0) return(empty_breaks())
 
     breaks <- min_x + range_x * proportions
-    breaks <- create_lr_breaks(breaks, left, close_end)
+    breaks <- create_lr_breaks(breaks, left)
 
     scaled_endpoints <- proportions
-    needs <- needs_extend(breaks, x, extend)
+    needs <- needs_extend(breaks, x, extend, left, close_end)
     if ((needs & LEFT) > 0) scaled_endpoints <- c(0, scaled_endpoints)
     if ((needs & RIGHT) > 0) scaled_endpoints <- c(scaled_endpoints, 1)
-    breaks <- maybe_extend(breaks, x, extend)
+    breaks <- extend_and_close(breaks, x, extend, left, close_end)
+
     attr(breaks, "scaled_endpoints") <- scaled_endpoints
+    names(breaks) <- names(scaled_endpoints)
 
     breaks
   }
@@ -313,22 +253,28 @@ brk_n <- function (n) {
   assert_that(is.count(n))
 
   function (x, extend, left, close_end) {
-    xs <- sort(x) # remove NAs
+    xs <- sort(x, decreasing = ! left, na.last = NA) # remove NAs
     if (length(xs) < 1L) return(empty_breaks())
 
     breaks <-  xs[c(seq(1L, length(xs), n), length(xs))]
+    if (! left) breaks <- rev(breaks)
     s1tons <- singletons(breaks)
     # gets rid of the first of every "triplet", including overlapping triplets:
     illegal <- which(s1tons[-1] & s1tons[-length(s1tons)])
     if (length(illegal) > 0) breaks <- breaks[-illegal]
-    breaks <- create_lr_breaks(breaks, left, close_end)
-    breaks <- maybe_extend(breaks, x, extend)
+    breaks <- create_extended_breaks(breaks, x, extend, left, close_end)
 
     breaks
   }
 }
 
 
+#' @param breaks A numeric vector.
+#' @name breaks-doc
+#' @return A function which returns an object of class `breaks`.
+NULL
+
+
 #' Create a standard set of breaks
 #'
 #' @inherit breaks-doc params return
@@ -343,18 +289,11 @@ brk_default <- function (breaks) {
   assert_that(noNA(breaks))
 
   function (x, extend, left, close_end) {
-    breaks <- create_lr_breaks(breaks, left, close_end)
-    maybe_extend(breaks, x, extend)
+    create_extended_breaks(breaks, x, extend, left, close_end)
   }
 }
 
 
-#' @param breaks A numeric vector.
-#' @name breaks-doc
-#' @return A (function which returns an) object of class `breaks`.
-NULL
-
-
 #' Create a `breaks` object manually
 #'
 #' @param breaks A vector, which must be sorted.
@@ -402,12 +341,35 @@ brk_manual <- function (breaks, left_vec) {
           noNA(left_vec),
           length(left_vec) == length(breaks)
         )
-  breaks <- create_breaks(breaks, left_vec)
+
 
   function (x, extend, left, close_end) {
     if (! left) warning("Ignoring `left` with `brk_manual()`")
     if (close_end) warning("Ignoring `close_end` with `brk_manual()`")
-    maybe_extend(breaks, x, extend)
+    breaks <- create_breaks(breaks, left_vec)
+    breaks <- extend_and_close(breaks, x, extend, left, close_end)
+  }
+}
+
+#' @rdname chop_fn
+#' @export
+#' @order 2
+brk_fn <- function (fn, ...) {
+  assert_that(is.function(fn))
+
+  function (x, extend, left, close_end) {
+    breaks <- fn(x, ...)
+    # some functions (e.g. quantile()) return a named vector
+    # which might create surprise labels:
+    breaks <- unname(breaks)
+    assert_that(is.numeric(breaks))
+    if (length(breaks) == 0) {
+      return(empty_breaks())
+    }
+
+    breaks <- create_extended_breaks(breaks, x, extend, left, close_end)
+
+    breaks
   }
 }
 
@@ -438,6 +400,7 @@ print.breaks <- function (x, ...) cat(format(x, ...))
 #' @export
 is.breaks <- function (x, ...) inherits(x, "breaks")
 
+
 on_failure(is.breaks) <- function (call, env) {
   paste0(deparse(call$x), " is not an object of class `breaks`")
 }
